"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.makeTypes = exports.createLiteralNodeFromProperties = void 0;
var typescript_1 = __importDefault(require("typescript"));
var common_1 = require("./common");
function schemaObjectTypeToArrayType(item) {
    return (0, common_1.appendNullToUnion)((0, common_1.nonArraySchemaObjectTypeToTs)(item), item.nullable);
}
function resolveArray(item) {
    if ((0, common_1.isReferenceObject)(item.items)) {
        return (0, common_1.appendNullToUnion)(typescript_1["default"].factory.createArrayTypeNode((0, common_1.createTypeRefFromRef)(item.items)), item.nullable);
    }
    if (item.items.properties) {
        return typescript_1["default"].factory.createArrayTypeNode(createLiteralNodeFromProperties(item.items));
    }
    return typescript_1["default"].factory.createArrayTypeNode((0, common_1.isArraySchemaObject)(item.items)
        ? resolveArray(item.items)
        : schemaObjectTypeToArrayType(item.items));
}
function resolveType(item) {
    if ((0, common_1.isReferenceObject)(item)) {
        return (0, common_1.createTypeRefFromRef)(item);
    }
    if ((0, common_1.isArraySchemaObject)(item)) {
        return resolveArray(item);
    }
    return item.type
        ? (0, common_1.nonArraySchemaObjectTypeToTs)(item)
        : createTypeAliasDeclarationType(item);
}
function createPropertySignature(item, name, required) {
    return typescript_1["default"].factory.createPropertySignature(
    /*modifiers*/ undefined, 
    /*name*/ typescript_1["default"].factory.createIdentifier(name), 
    /*questionToken*/ required
        ? undefined
        : typescript_1["default"].factory.createToken(typescript_1["default"].SyntaxKind.QuestionToken), 
    /*type*/ resolveType(item));
}
function createPropertySignatures(item) {
    if (!item.properties) {
        return [];
    }
    return Object.entries(item.properties).map(function (_a) {
        var _b, _c;
        var _d = __read(_a, 2), name = _d[0], prop = _d[1];
        return createPropertySignature(prop, name, (_c = (_b = item.required) === null || _b === void 0 ? void 0 : _b.includes(name)) !== null && _c !== void 0 ? _c : false);
    });
}
function createTypeAliasDeclarationTypeWithSchemaObject(item) {
    if ((0, common_1.isAllOfObject)(item) && item.allOf) {
        return typescript_1["default"].factory.createIntersectionTypeNode(item.allOf.map(function (allOfItem) {
            return (0, common_1.isReferenceObject)(allOfItem)
                ? (0, common_1.createTypeRefFromRef)(allOfItem)
                : createTypeAliasDeclarationTypeWithSchemaObject(allOfItem);
        }));
    }
    if ((0, common_1.isOneOfOrAnyOfObject)(item)) {
        var items = item.oneOf || item.anyOf;
        if (items) {
            return typescript_1["default"].factory.createUnionTypeNode(items.map(function (oneOrAnyItem) {
                return (0, common_1.isReferenceObject)(oneOrAnyItem)
                    ? (0, common_1.createTypeRefFromRef)(oneOrAnyItem)
                    : createTypeAliasDeclarationTypeWithSchemaObject(oneOrAnyItem);
            }));
        }
    }
    if ((0, common_1.isArraySchemaObject)(item)) {
        return typescript_1["default"].factory.createArrayTypeNode((0, common_1.isReferenceObject)(item.items)
            ? (0, common_1.createTypeRefFromRef)(item.items)
            : createTypeAliasDeclarationTypeWithSchemaObject(item.items));
    }
    if (item.additionalProperties) {
        return (0, common_1.createDictionaryType)(item);
    }
    if (item.properties) {
        return createLiteralNodeFromProperties(item);
    }
    return (0, common_1.nonArraySchemaObjectTypeToTs)(item);
}
function createLiteralNodeFromProperties(item) {
    return typescript_1["default"].factory.createTypeLiteralNode(createPropertySignatures(item));
}
exports.createLiteralNodeFromProperties = createLiteralNodeFromProperties;
function createTypeAliasDeclarationType(item) {
    return (0, common_1.isReferenceObject)(item)
        ? (0, common_1.createTypeRefFromRef)(item)
        : createTypeAliasDeclarationTypeWithSchemaObject(item);
}
function makeTypes(doc) {
    var _a;
    var schemas = (_a = doc.components) === null || _a === void 0 ? void 0 : _a.schemas;
    if (!schemas) {
        return [];
    }
    return Object.entries(schemas).map(function (_a) {
        var _b = __read(_a, 2), schemaName = _b[0], item = _b[1];
        return typescript_1["default"].factory.createTypeAliasDeclaration(
        /*decorators*/ undefined, 
        /*modifiers*/ [typescript_1["default"].factory.createModifier(typescript_1["default"].SyntaxKind.ExportKeyword)], 
        /*name*/ typescript_1["default"].factory.createIdentifier((0, common_1.sanitizeTypeName)(schemaName)), 
        /*typeParameters*/ undefined, 
        /*type*/ createTypeAliasDeclarationType(item));
    });
}
exports.makeTypes = makeTypes;
